import hashlib
import json
import re
import sys

try:
    from urllib.request import urlopen, Request  # Python 3
except ImportError:
    from urllib2 import urlopen, Request  # Python 2
from .Acme import Acme
from .DigitalOcean import DigitalOcean
from .AWSRoute53 import AWSRoute53
from .Cloudflare import Cloudflare


class AcmeV2(Acme):
    def register_account(self):
        """
        Generate new 2049 bit account key, domain key if not generated
        Register account key with ACME
        Return:
             dict, directory data from acme server
        """
        try:
            self.log.info("trying to create account key {0}".format(self.account_key))
            self.create_key(self.account_key)
        except Exception as e:
            self.log.error("creating key {0} {1}".format(type(e).__name__, e))
            sys.exit(1)
        # directory is needed later for order placement
        directory = (
            urlopen(
                Request(self.api_url, headers={"Content-Type": "application/jose+json"})
            )
            .read()
            .decode("utf8")
        )
        directory = json.loads(directory)
        directory["_kid"] = None
        self.log.info("trying to register acmev2 account")
        payload = {"termsOfServiceAgreed": True}
        code, result, headers = self._send_signed_request(
            url=directory["newAccount"], payload=payload, directory=directory
        )
        if code == 201:
            self.log.info("registered!")
        elif code == 200:
            self.log.info("already registered")
        else:
            self.log.error(
                "error registering: {0} {1} {2}".format(code, result, headers)
            )
            sys.exit(1)
        directory["_kid"] = headers["Location"]
        try:
            self.log.info("trying to create domain key")
            self.create_key(self.domain_key)
        except Exception as e:
            self.log.error("creating key {0} {1}".format(type(e).__name__, e))
            sys.exit(1)
        return directory

    def _sign_certificate(self, order, directory):
        """Sign certificate"""
        self.log.info("signing certificate")
        csr = self.create_csr()
        payload = {"csr": self._b64(csr)}
        code, result, headers = self._send_signed_request(
            url=order["finalize"], payload=payload, directory=directory
        )
        self.log.debug("{0}, {1}, {2}".format(code, result, headers))
        if code > 399:
            self.log.error("error signing certificate: {0} {1}".format(code, result))
            self._reload_nginx()
            sys.exit(1)
        self.log.info("certificate signed!")
        self.log.info("downloading certificate")
        certificate_url = json.loads(result)["certificate"]
        certificate_pem = self._send_signed_request(certificate_url, "", directory)[1]
        self.log.info("writing result file in {0}".format(self.cert_path))
        try:
            with open(self.cert_path, "w") as fd:
                fd.write(certificate_pem)
        except Exception as e:
            self.log.error("error writing cert: {0} {1}".format(type(e).__name__, e))
        if not self.skip_nginx_reload:
            self._reload_nginx()

    def solve_http_challenge(self, directory):
        """
        Solve HTTP challenge
        Params:
            directory, dict, directory data from acme server
        """
        self.log.info("acmev2 http challenge")
        self.log.info("preparing new order")
        order_payload = {
            "identifiers": [{"type": "dns", "value": d} for d in self.domains]
        }
        code, order, _ = self._send_signed_request(
            url=directory["newOrder"], payload=order_payload, directory=directory
        )
        self.log.debug(order)
        order = json.loads(order)
        self.log.info("order created")
        for url in order["authorizations"]:
            order_data = self._send_signed_request(url, "", directory)
            auth = json.loads(order_data[1])
            self.log.debug(json.dumps(auth))
            domain = auth["identifier"]["value"]
            self.log.info("verifying domain {0}".format(domain))
            challenge = self._get_challenge(auth["challenges"], "http-01")
            token = re.sub(r"[^A-Za-z0-9_\-]", "_", challenge["token"])
            thumbprint = self._thumbprint()
            self.log.info("adding nginx virtual host and completing challenge")
            try:
                challenge_dir = self._write_vhost()
                self._write_challenge(challenge_dir, token, thumbprint)
            except Exception as e:
                self.log.error(
                    "error adding virtual host {0} {1}".format(type(e).__name__, e)
                )
                sys.exit(1)
            self.log.info("asking acme server to verify challenge")
            code, result, _ = self._send_signed_request(
                url=challenge["url"], directory=directory
            )
            try:
                if code > 399:
                    self.log.error(
                        "error triggering challenge: {0} {1}".format(code, result)
                    )
                    sys.exit(1)
                self._verify_challenge(url, domain, directory)
            finally:
                self._cleanup(
                    ["{0}/{1}".format(challenge_dir, token), self.vhost, challenge_dir]
                )
                self._reload_nginx()
        self._sign_certificate(order, directory)

    def solve_dns_challenge(self, directory, client):
        """
        Solve DNS challenge
        Params:
            directory, dict, directory data from acme server
            client, object, dns provider client implementation
        """
        self.log.info("acmev2 dns challenge")
        self.log.info("preparing new order")
        order_payload = {
            "identifiers": [{"type": "dns", "value": d} for d in self.domains]
        }
        code, order, _ = self._send_signed_request(
            url=directory["newOrder"], payload=order_payload, directory=directory
        )
        self.log.debug(order)
        order = json.loads(order)
        self.log.info("order created")
        for url in order["authorizations"]:
            order_data = self._send_signed_request(url, "", directory)
            auth = json.loads(order_data[1])
            self.log.debug(json.dumps(auth))
            domain = auth["identifier"]["value"]
            self.log.info("verifying domain {0}".format(domain))
            challenge = self._get_challenge(auth["challenges"], "dns-01")
            token = re.sub(r"[^A-Za-z0-9_\-]", "_", challenge["token"])
            thumbprint = self._thumbprint()
            keyauthorization = "{0}.{1}".format(token, thumbprint)
            txt_record = self._b64(
                hashlib.sha256(keyauthorization.encode("utf8")).digest()
            )
            self.log.info(
                "creating TXT dns record _acme-challenge.{0} IN TXT {1}".format(
                    domain, txt_record
                )
            )
            try:
                record = client.create_record(
                    domain=domain,
                    name="_acme-challenge.{0}.".format(domain.lstrip("*.").rstrip(".")),
                    data=txt_record,
                )
            except Exception as e:
                self.log.error("error creating dns record")
                self.log.error(e)
                sys.exit(1)
            try:
                self.log.info("asking acme server to verify challenge")
                payload = {"keyAuthorization": keyauthorization}
                code, result, headers = self._send_signed_request(
                    url=challenge["url"], payload=payload, directory=directory
                )
                if code > 399:
                    self.log.error(
                        "error triggering challenge: {0} {1}".format(code, result)
                    )
                    raise Exception(result)
                self._verify_challenge(url, domain, directory)
            finally:
                try:
                    if not self.debug:
                        self.log.info("delete dns record")
                        client.delete_record(domain=domain, record=record)
                except Exception as e:
                    self.log.error("error deleting dns record")
                    self.log.error(e)
        self._sign_certificate(order, directory)

    def get_certificate(self):
        directory = self.register_account()
        if self.dns_provider:
            if self.dns_provider == "digitalocean":
                dns_client = DigitalOcean()
            elif self.dns_provider == "route53":
                dns_client = AWSRoute53()
            elif self.dns_provider == "cloudflare":
                dns_client = Cloudflare()
            self.solve_dns_challenge(directory, dns_client)
        else:
            self.solve_http_challenge(directory)



import json
from os import getenv

try:
    from urllib.request import urlopen, Request  # Python 3
    from urllib.error import HTTPError
except ImportError:
    from urllib2 import urlopen, Request  # Python 2
    from urllib2 import HTTPError


class Cloudflare(object):
    def __init__(self):
        self.token = getenv("API_TOKEN")
        self.api = "https://api.cloudflare.com/client/v4"
        if not self.token:
            raise Exception("API_TOKEN not found in environment")

    def determine_domain(self, domain):
        """Determine registered domain in API
        Returns zone id
        """
        request_headers = {
            "Content-Type": "application/json",
            "Authorization": "Bearer {0}".format(self.token),
        }
        api = "{0}/zones?name={1}".format(self.api, domain)
        response = urlopen(Request(api, headers=request_headers))
        if response.getcode() != 200:
            raise Exception(json.loads(response.read().decode("utf8")))
        domains = json.loads(response.read().decode("utf8"))["result"]
        for d in domains:
            if d["name"] in domain:
                return d["id"]

    def create_record(self, name, data, domain):
        """
        Create DNS record
        Params:
            name, string, record name
            data, string, record data
            domain, string, dns domain
        Return:
            record_id, string, created record id
        """
        zone_id = self.determine_domain(domain)
        api = "{0}/zones/{1}/dns_records".format(self.api, zone_id)
        request_headers = {
            "Content-Type": "application/json",
            "Authorization": "Bearer {0}".format(self.token),
        }
        request_data = {
            "type": "TXT",
            "ttl": 300,
            "name": name,
            "content": data,
            "proxied": False,
        }
        try:
            response = urlopen(
                Request(
                    api,
                    data=json.dumps(request_data).encode("utf8"),
                    headers=request_headers,
                )
            )
        except HTTPError as e:
            raise Exception(e.read().decode("utf8"))
        if response.getcode() != 200:
            raise Exception(json.loads(response.read().decode("utf8")))
        return json.loads(response.read().decode("utf8"))["result"]["id"]

    def delete_record(self, record, domain):
        """
        Delete DNS record
        Params:
            record, string, record id number
            domain, string, dns domain
        """
        zone_id = self.determine_domain(domain)
        api = "{0}/zones/{1}/dns_records/{2}".format(self.api, zone_id, record)
        request_headers = {
            "Content-Type": "application/json",
            "Authorization": "Bearer {0}".format(self.token),
        }
        request = Request(
            api, data=json.dumps({}).encode("utf8"), headers=request_headers
        )
        # this is hack around urllib to send DELETE request
        request.get_method = lambda: "DELETE"
        try:
            response = urlopen(request)
        except HTTPError as e:
            raise Exception(e.read().decode("utf8"))
        if response.getcode() != 200:
            raise Exception(json.loads(response.read().decode("utf8")))



import base64
import json
import re
import sys
import textwrap

try:
    from urllib.request import urlopen  # Python 3
except ImportError:
    from urllib2 import urlopen  # Python 2
from .Acme import Acme


class AcmeV1(Acme):
    def register_account(self):
        """
        Generate new 2049 bit account key, domain key if not generated
        Register account key with ACME
        Return:
             dict, directory data from acme server
        """
        try:
            self.log.info("trying to create account key {0}".format(self.account_key))
            account_key = self.create_key(self.account_key)
        except Exception as e:
            self.log.error("creating key {0} {1}".format(type(e).__name__, e))
            sys.exit(1)
        self.log.info("trying to register acmev1 account")
        payload = {
            "resource": "new-reg",
            "agreement": "https://letsencrypt.org/documents/LE-SA-v1.2-November-15-2017.pdf",
        }
        code, result, headers = self._send_signed_request(
            url=self.api_url + "/acme/new-reg", payload=payload
        )
        if code == 201:
            self.log.info("registered!")
        elif code == 409:
            self.log.info("already registered")
        else:
            self.log.error(
                "error registering: {0} {1} {2}".format(code, result, headers)
            )
            sys.exit(1)
        return account_key

    def get_certificate(self):
        # Generate new 2048 bit account, domain private keys only if not set
        self.register_account()
        try:
            self.create_key(self.domain_key)
        except Exception as e:
            self.log.error("creating key {0} {1}".format(type(e).__name__, e))
            sys.exit(1)
        csr = self.create_csr()
        # Solve challenge
        self.log.info("acmev1 http challenge")
        for domain in self.domains:
            self.log.info("requesting challenge for {0}".format(domain))
            payload = {
                "resource": "new-authz",
                "identifier": {"type": "dns", "value": domain},
            }
            code, result, _ = self._send_signed_request(
                url=self.api_url + "/acme/new-authz", payload=payload
            )
            if code != 201:
                self.log.error(
                    "error requesting challenges: {0} {1}".format(code, result)
                )
                sys.exit(1)
            challenge = self._get_challenge(json.loads(result)["challenges"], "http-01")
            token = re.sub(r"[^A-Za-z0-9_\-]", "_", challenge["token"])
            thumbprint = self._thumbprint()
            self.log.info("adding nginx virtual host and completing challenge")
            try:
                challenge_dir = self._write_vhost()
                self._write_challenge(challenge_dir, token, thumbprint)
            except Exception as e:
                self.log.error(
                    "error adding virtual host {0} {1}".format(type(e).__name__, e)
                )
                sys.exit(1)
            self.log.info(
                "asking acme server to verify challenge for {0}".format(domain)
            )
            try:
                payload = {
                    "resource": "challenge",
                    "keyAuthorization": "{0}.{1}".format(token, thumbprint),
                }
                code, result, _ = self._send_signed_request(
                    url=challenge["uri"], payload=payload
                )
                if code != 202:
                    self.log.error(
                        "error triggering challenge: {0} {1}".format(code, result)
                    )
                    sys.exit(1)
                self._verify_challenge(challenge["uri"], domain)
            finally:
                self._cleanup(
                    ["{0}/{1}".format(challenge_dir, token), self.vhost, challenge_dir]
                )
                self._reload_nginx()
        self.log.info("signing certificate")
        try:
            payload = {"resource": "new-cert", "csr": self._b64(csr)}
            code, result, _ = self._send_signed_request(
                url=self.api_url + "/acme/new-cert", payload=payload
            )
            if code != 201:
                self.log.error(
                    "error signing certificate: {0} {1}".format(code, result)
                )
                sys.exit(1)
            self.log.info("certificate signed!")
            try:
                self.log.info("getting chain from {0}".format(self.chain))
                chain_str = urlopen(self.chain).read()
                if chain_str:
                    chain_str = chain_str.decode("utf8")
            except Exception as e:
                self.log.error(
                    "error getting chain: {0} {1}".format(type(e).__name__, e)
                )
                sys.exit(1)
            self.log.info("writing result file in {0}".format(self.cert_path))
            try:
                with open(self.cert_path, "w") as fd:
                    fd.write(
                        """-----BEGIN CERTIFICATE-----\n{0}\n-----END CERTIFICATE-----\n""".format(
                            "\n".join(
                                textwrap.wrap(
                                    base64.b64encode(result).decode("utf8"), 64
                                )
                            )
                        )
                    )
                    fd.write(chain_str)
            except Exception as e:
                self.log.error(
                    "error writing cert: {0} {1}".format(type(e).__name__, e)
                )
                sys.exit(1)
        finally:
            self._reload_nginx()



import Crypto.PublicKey.RSA
import OpenSSL
import base64
import binascii
import hashlib
import json
import os
import subprocess
import sys
import tempfile
import time
from datetime import datetime, timedelta

try:
    from urllib.request import urlopen, Request  # Python 3
except ImportError:
    from urllib2 import urlopen, Request  # Python 2


__version__ = "0.3.2"


class Acme(object):
    def __init__(
        self,
        api_url,
        logger,
        domains=None,
        vhost="/etc/nginx/sites-enabled/0-letsencrypt.conf",
        account_key="/etc/ssl/private/letsencrypt-account.key",
        domain_key="/etc/ssl/private/letsencrypt-domain.key",
        cert_path="/etc/ssl/private/letsencrypt-domain.pem",
        dns_provider=None,
        skip_nginx_reload=False,
        renew_days=None,
        debug=False,
    ):
        """
        Params:
            api_url, str, Letsencrypt API URL
            logger, logging object
            debug, bool, if debug mode enabled
            domains, list, list with domain names for certificate
            vhost, str, path to nginx virtual host config
            account_key, str, path to letsencrypt account key
            domain_key, str, path to certificate private key
            cert_path, str, path to output certificate file
            dns_provider, list, dns provider that is used for dns challenge
            skip_nginx_reload, bool, should nginx be reloaded after certificate issue
        """
        self.debug = debug
        if not domains:
            domains = list()
        self.domains = domains
        self.vhost = vhost
        self.account_key = account_key
        self.domain_key = domain_key
        self.cert_path = cert_path
        self.api_url = api_url
        self.log = logger
        # LetsEncrypt Root CA certificate chain, needed for ACMEv1
        self.chain = "https://letsencrypt.org/certs/lets-encrypt-x3-cross-signed.pem"
        self.dns_provider = dns_provider
        self.skip_nginx_reload = skip_nginx_reload
        self.renew_days = renew_days

        self.IsOutOfDate = True
        if self.renew_days:
            try:
                cert = OpenSSL.crypto.load_certificate(
                    OpenSSL.crypto.FILETYPE_PEM, open(self.cert_path).read()
                )
                date_format, encoding = "%Y%m%d%H%M%SZ", "ascii"
                not_before = datetime.strptime(
                    cert.get_notBefore().decode(encoding), date_format
                )
                not_after = datetime.strptime(
                    cert.get_notAfter().decode(encoding), date_format
                )
                now = datetime.now()
                self.log.debug(
                    "x509: {0}, not_before: {1}, not_after: {2}".format(
                        cert, not_before, not_after
                    )
                )
                certTimeThreshold = not_after - timedelta(days=self.renew_days)

                self.IsOutOfDate = (
                    (not_before > now) or (not_after < now) or (certTimeThreshold < now)
                )
                self.log.info(
                    "Cert file {1} (expiration time {0})".format(
                        certTimeThreshold,
                        "is out of date" if self.IsOutOfDate else "is not out of date",
                    )
                )

            except OSError as e:
                if e.errno == 2:
                    self.log.info(
                        "Cert file {0} not found -> DO UPDATE CERT".format(
                            self.cert_path
                        )
                    )
            except Exception:
                pass

    def _reload_nginx(self):
        """Reload nginx"""
        self.log.info("running nginx -s reload")
        process = subprocess.Popen(
            "nginx -s reload".split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        process_out = process.communicate()
        self.log.debug(process_out[0])
        self.log.debug(process_out[1])
        if process.returncode > 0:
            self.log.error("failed to reload nginx")
            self.log.error(process_out[1])

    def _write_vhost(self):
        """Write virtual host configuration for http"""
        challenge_file = tempfile.mkdtemp()
        self.log.info("created challenge file into {0}".format(challenge_file))
        os.chmod(challenge_file, 0o777)
        vhost_data = """
server {{
    listen 80;
    listen [::]:80;
    server_name {domain};
    location /.well-known/acme-challenge/ {{
        alias {alias}/;
        try_files $uri =404;
    }}
}}""".format(
            domain=" ".join(self.domains), alias=challenge_file
        )
        self.log.info("writing virtual host into {0}".format(self.vhost))
        with open(self.vhost, "w") as fd:
            fd.write(vhost_data)
        os.chmod(self.vhost, 0o644)
        self._reload_nginx()
        return challenge_file

    def _write_challenge(self, challenge_dir, token, thumbprint):
        self.log.info("writing challenge file into {0}".format(self.vhost))
        with open("{0}/{1}".format(challenge_dir, token), "w") as fd:
            fd.write("{0}.{1}".format(token, thumbprint))

    def create_key(self, key_path, key_type=OpenSSL.crypto.TYPE_RSA, bits=2048):
        """
        Return created private key and writes it into key_path
        Params:
            key_path, str, writable path for key
            key_type, int, SSL key type
            bits, int, the number of bits of the key
        Return:
            string with private key
        """
        try:
            with open(key_path, "r") as fd:
                private_key = fd.read()
        except IOError:
            key = OpenSSL.crypto.PKey()
            key.generate_key(key_type, bits)
            private_key = OpenSSL.crypto.dump_privatekey(
                OpenSSL.crypto.FILETYPE_PEM, key
            )
            self.log.info(
                "can not open key, writing new in {path}".format(path=key_path)
            )
            if not os.path.isdir(os.path.dirname(key_path)):
                os.mkdir(os.path.dirname(key_path))
            with open(key_path, "wb") as fd:
                fd.write(private_key)
            os.chmod(key_path, 0o400)
        return private_key

    def create_csr(self):
        """Generate CSR
        Return:
            string with CSR in DER format
        """
        sna = ", ".join(["DNS:{0}".format(i) for i in self.domains]).encode("utf8")
        req = OpenSSL.crypto.X509Req()
        req.get_subject().CN = self.domains[0]
        req.add_extensions(
            [
                OpenSSL.crypto.X509Extension(
                    "subjectAltName".encode("utf8"), critical=False, value=sna
                )
            ]
        )
        with open(self.domain_key, "r") as fd:
            key = fd.read()
        pk = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
        req.set_pubkey(pk)
        req.set_version(2)
        req.sign(pk, "sha256")
        return OpenSSL.crypto.dump_certificate_request(
            OpenSSL.crypto.FILETYPE_ASN1, req
        )

    @staticmethod
    def _b64(b):
        return base64.urlsafe_b64encode(b).decode("utf8").replace("=", "")

    def _sign_message(self, message):
        """
        Sign provided message with key
        Params:
            message, str, message to sign
        Return:
            string with signed message
        """
        with open(self.account_key, "r") as fd:
            key = fd.read()
        pk = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
        return OpenSSL.crypto.sign(pk, message.encode("utf8"), "sha256")

    def _jws(self):
        """Return JWS dict from string account key"""
        with open(self.account_key, "r") as fd:
            key = fd.read()
        pk = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, key)
        pk_asn1 = OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_ASN1, pk)
        k = Crypto.PublicKey.RSA.importKey(pk_asn1)
        # private key public exponent in hex format
        exponent = "{0:x}".format(k.e)
        exponent = "0{0}".format(exponent) if len(exponent) % 2 else exponent
        # private key modulus in hex format
        modulus = "{0:x}".format(k.n)
        header = {
            "alg": "RS256",
            "jwk": {
                "e": self._b64(binascii.unhexlify(exponent.encode("utf8"))),
                "kty": "RSA",
                "n": self._b64(binascii.unhexlify(modulus.encode("utf8"))),
            },
        }
        return header

    def _thumbprint(self):
        """Return account thumbprint"""
        accountkey_json = json.dumps(
            self._jws()["jwk"], sort_keys=True, separators=(",", ":")
        )
        return self._b64(hashlib.sha256(accountkey_json.encode("utf8")).digest())

    def _cleanup(self, files):
        if not self.debug:
            for f in files:
                self.log.info("removing {0}".format(f))
                try:
                    if os.path.isdir(f):
                        os.rmdir(f)
                    else:
                        os.remove(f)
                except OSError:
                    self.log.debug("{0} does not exist".format(f))

    def _send_signed_request(self, url, payload=None, directory=None):
        """
        Send signed request to ACME CA
        Params:
            url, str, url for request
            payload, any type, any payload you want to send, usually dict
            directory, dict, directory data from acme server
        """
        request_headers = {
            "Content-Type": "application/jose+json",
            "User-Agent": "acme-nginx/{0} urllib".format(self.version()),
        }
        if payload is None:
            payload = {}
        # on POST-as-GET, final payload has to be just empty string
        if payload == "":
            payload64 = ""
        else:
            payload64 = self._b64(json.dumps(payload).encode("utf8"))
        # If not set then ACMEv1 is used
        if directory:
            # jwk and kid header fields are mutually exclusive
            if directory["_kid"]:
                protected = {"kid": directory["_kid"]}
            else:
                protected = self._jws()
            protected["nonce"] = urlopen(
                Request(directory["newNonce"], headers=request_headers)
            ).headers["Replay-Nonce"]
            protected["url"] = url
            protected["alg"] = "RS256"  # set for compatibility
        else:
            protected = self._jws()
            protected["nonce"] = urlopen(self.api_url + "/directory").headers[
                "Replay-Nonce"
            ]
        protected64 = self._b64(json.dumps(protected).encode("utf8"))
        signature = self._sign_message("{0}.{1}".format(protected64, payload64))
        data = json.dumps(
            {
                "protected": protected64,
                "payload": payload64,
                "signature": self._b64(signature),
            }
        )
        try:
            resp = urlopen(
                Request(
                    url,
                    data=data.encode("utf8"),
                    headers=request_headers,
                    method="POST",
                )
            )
            resp_data = resp.read()
            try:
                resp_data = resp_data.decode("utf8")
            except UnicodeDecodeError:
                pass
            return resp.getcode(), resp_data, resp.headers
        except Exception as e:
            return (
                getattr(e, "code", None),
                getattr(e, "read", e.__str__)(),
                getattr(e, "headers", None),
            )

    def _verify_challenge(self, url, domain, directory=None):
        """Verify challenge for domain"""
        self.log.info("waiting for {0} challenge verification".format(domain))
        checks_count = 60
        while True:
            checks_count -= 1
            if checks_count <= 0:
                self.log.error("reached waiting limit")
                sys.exit(1)
            challenge_status = json.loads(
                self._send_signed_request(url, "", directory)[1]
            )
            if challenge_status["status"] == "pending":
                time.sleep(5)
                continue
            elif challenge_status["status"] == "valid":
                self.log.info("{0} verified!".format(domain))
                break
            self.log.error(
                "{0} challenge did not pass: {1}".format(domain, challenge_status)
            )
            sys.exit(1)

    @staticmethod
    def _get_challenge(challenges, challenge_type):
        """
        Return challenge from dict
        Params:
            challenge, dict, challenge data structure from acme api
            challenge_type, str, challenge type
        Return:
            challenge key
        """
        for challenge in challenges:
            if challenge["type"] == challenge_type:
                return challenge

    @staticmethod
    def version():
        return __version__



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

p_version = "0.3.2"

with open("README.md") as f:
    long_description = f.read()

setup(
    name="acme-nginx",
    version=p_version,
    author="Konstantin Shcherban",
    author_email="k.scherban@gmail.com",
    packages=find_packages(),
    url="https://github.com/kshcherban/acme-nginx",
    download_url="https://github.com/kshcherban/acme-nginx/tarball/v{0}".format(
        p_version
    ),
    license="GPL v3",
    description="A simple client/tool for Let's Encrypt or any ACME server that issues SSL certificates.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[
        "tls",
        "ssl",
        "certificate",
        "acme",
        "letsencrypt",
        "nginx",
        "wildcard certificate",
        "wildcard",
    ],
    install_requires=[
        "pyOpenSSL~=20.0",
        "pycryptodome~=3.14",
        "boto3~=1.17",
    ],
    entry_points={
        "console_scripts": [
            "acme-nginx = acme_nginx.client:main",
        ]
    },
)
